"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "FederatedTypesPlugin", {
    enumerable: true,
    get: ()=>FederatedTypesPlugin
});
const _interopRequireDefault = require("@swc/helpers/lib/_interop_require_default.js").default;
const _typescript = /*#__PURE__*/ _interopRequireDefault(require("typescript"));
const _download = /*#__PURE__*/ _interopRequireDefault(require("download"));
const _fs = /*#__PURE__*/ _interopRequireDefault(require("fs"));
const _path = /*#__PURE__*/ _interopRequireDefault(require("path"));
const _lodashGet = /*#__PURE__*/ _interopRequireDefault(require("lodash.get"));
const _axios = /*#__PURE__*/ _interopRequireDefault(require("axios"));
let FederatedTypesPlugin = class FederatedTypesPlugin {
    apply(compiler) {
        var ref, ref1;
        let recompileInterval;
        this.webpackCompilerOptions = compiler.options;
        const distPath = (0, _lodashGet.default)(this.webpackCompilerOptions, 'devServer.static.directory') || (0, _lodashGet.default)(this.webpackCompilerOptions, 'output.path') || 'dist';
        this.distDir = _path.default.join(distPath, this.typescriptFolderName);
        this.typesIndexJsonFilePath = _path.default.join(this.distDir, this.typesIndexJsonFileName);
        // The '/' at the end is necessary for TS Compiler to emit files to the output dir.
        this.tsCompilerOptions.outDir = _path.default.join(this.distDir, '/');
        const federationOptions = this.webpackCompilerOptions.plugins.find((plugin)=>{
            return plugin.constructor.name === 'ModuleFederationPlugin';
        });
        const inheritedPluginOptions = (0, _lodashGet.default)(federationOptions, '_options') || null;
        this.exposedComponents = ((ref = this.options) == null ? void 0 : ref.exposes) || inheritedPluginOptions.exposes;
        this.remoteComponents = ((ref1 = this.options) == null ? void 0 : ref1.remotes) || inheritedPluginOptions.remotes;
        compiler.hooks.afterCompile.tap('FederatedTypes', (compilation)=>{
            // Reset and create an Interval to refetch types every 60 seconds
            clearInterval(recompileInterval);
            if (compiler.options.mode === 'development') {
                recompileInterval = setInterval(()=>{
                    this.run(compilation);
                }, 1000 * 60);
            }
            // Runs a compilation immediately
            this.run(compilation);
        });
    }
    run(compilation) {
        if (this.exposedComponents) {
            this.extractTypes();
        }
        // Time to import the remote types
        if (this.remoteComponents) {
            // Get the remote URL origin
            this.importRemoteTypes();
        }
    }
    importRemoteTypes() {
        if (this.remoteComponents) {
            const remoteUrls = Object.entries(this.remoteComponents).map(([remote, entry])=>{
                const remoteUrl = entry.substring(0, entry.lastIndexOf('/'));
                const [, url] = remoteUrl.split('@');
                return {
                    origin: url != null ? url : remoteUrl,
                    remote
                };
            });
            remoteUrls.forEach(({ origin , remote  })=>{
                _axios.default.get(`${origin}/${this.typescriptFolderName}/${this.typesIndexJsonFileName}`).then((indexFileResp)=>{
                    var // Download all the d.ts files mentioned in the index file
                    ref;
                    (ref = indexFileResp.data) == null ? void 0 : ref.forEach((file)=>(0, _download.default)(`${origin}/${this.typescriptFolderName}/${file}`, `${this.typescriptFolderName}/${remote}`, {
                            filename: file
                        }));
                }).catch((e)=>console.log('ERROR fetching/writing types', e));
            });
        }
    }
    extractTypes() {
        if (this.exposedComponents) {
            const exposeSrcToDestMap = {};
            const normalizedFileNames = Object.entries(this.exposedComponents)// './Component': 'path/to/component' -> ['./Component', 'path/to/component']
            .map(([exposeDest, exposeSrc])=>{
                const [rootDir, entry] = exposeSrc.split(/\/(?=[^/]+$)/);
                const cwd = this.webpackCompilerOptions.context || process.cwd();
                const normalizedRootDir = _path.default.resolve(cwd, rootDir);
                const filenameWithExt = this.resolveFilenameWithExtension(normalizedRootDir, entry);
                const pathWithExt = _path.default.resolve(cwd, rootDir, filenameWithExt);
                exposeSrcToDestMap[pathWithExt] = exposeDest;
                return pathWithExt;
            }).filter((entry)=>/\.tsx?$/.test(entry));
            const host = _typescript.default.createCompilerHost(this.tsCompilerOptions);
            const originalWriteFileFn = host.writeFile;
            host.writeFile = (filename, text, writeOrderByteMark, onError, sourceFiles, data)=>{
                // for exposes: { "./expose/path": "path/to/file" }
                // force typescript to write compiled output to "@mf-typescript/expose/path"
                const newFileName = `${exposeSrcToDestMap[(sourceFiles == null ? void 0 : sourceFiles[0].fileName) || '']}.d.ts`;
                const newFilePath = _path.default.join(this.tsCompilerOptions.outDir, newFileName);
                this.tsDefinitionFilesObj[newFilePath] = text;
                originalWriteFileFn(newFilePath, text, writeOrderByteMark, onError, sourceFiles, data);
            };
            const program = _typescript.default.createProgram(normalizedFileNames, this.tsCompilerOptions, host);
            const emitResult = program.emit();
            if (!emitResult.emitSkipped) {
                const files = Object.keys(this.tsDefinitionFilesObj).map((file)=>file.slice(`${this.distDir}/`.length));
                _fs.default.writeFile(this.typesIndexJsonFilePath, JSON.stringify(files), (e)=>{
                    if (e) {
                        console.log('Error saving the types index', e);
                    }
                });
            }
        }
    }
    getFiles(dir) {
        // Simple caching mechanism to improve performance reading the file system
        if (this.fsCache.has(dir)) {
            return this.fsCache.get(dir);
        }
        const files = _fs.default.readdirSync(dir);
        this.fsCache.set(dir, files);
        return files;
    }
    resolveFilenameWithExtension(rootDir, entry) {
        // Check path exists and it's a directory
        if (!_fs.default.existsSync(rootDir) || !_fs.default.lstatSync(rootDir).isDirectory()) {
            throw new Error('rootDir must be a directory');
        }
        let filename;
        try {
            // Try to resolve exposed component using index
            const files = this.getFiles(_path.default.join(rootDir, entry));
            filename = files == null ? void 0 : files.find((file)=>file.split('.')[0] === 'index');
            if (!filename) {
                throw new Error(`filename ${filename} not found`);
            }
            return `${entry}/${filename}`;
        } catch (err) {
            const files1 = this.getFiles(rootDir);
            // Handle case where directory contains similar filenames
            // or where a filename like `Component.base.tsx` is used
            filename = files1 == null ? void 0 : files1.find((file)=>{
                const baseFile = _path.default.basename(file, _path.default.extname(file));
                const baseEntry = _path.default.basename(entry, _path.default.extname(entry));
                return baseFile === baseEntry;
            });
            if (!filename) {
                throw new Error(`filename ${filename} not found`);
            }
            return filename;
        }
    }
    constructor(options){
        this.fsCache = new Map();
        this.tsDefinitionFilesObj = {};
        this.typescriptFolderName = '@mf-typescript';
        this.typesIndexJsonFileName = '__types_index.json';
        this.options = options;
        this.tsCompilerOptions = {
            declaration: true,
            emitDeclarationOnly: true
        };
    }
};

//# sourceMappingURL=Typescript.js.map